@@ -18,7 +18,7 @@ from typing import Any, Callable, Coroutine, Optional, Type, TypeVar, final
 import aiohttp
 
 from .code_mappings import RoborockDockTypeCode
-from .command_cache import CacheableAttribute, CommandType, RoborockAttribute, create_cache_map, parse_method
+from .command_cache import CacheableAttribute, CommandType, RoborockAttribute, find_cacheable_attribute, get_cache_map
 from .containers import (
     ChildLockStatus,
     CleanRecord,
@@ -75,6 +75,11 @@ COMMANDS_SECURED = [
     RoborockCommand.GET_MULTI_MAP,
 ]
 RT = TypeVar("RT", bound=RoborockBase)
+WASH_N_FILL_DOCK = [
+    RoborockDockTypeCode.empty_wash_fill_dock,
+    RoborockDockTypeCode.s8_dock,
+    RoborockDockTypeCode.p10_dock,
+]
 
 
 def md5hex(message: str) -> str:
@@ -152,6 +157,9 @@ class AttributeCache:
         await self._async_value()
         return response
 
+    async def refresh_value(self):
+        await self._async_value()
+
 
 device_cache: dict[str, dict[CacheableAttribute, AttributeCache]] = {}
 
@@ -171,12 +179,11 @@ class RoborockClient:
         cache = device_cache.get(device_info.device.duid)
         if not cache:
             cache = {
-                cacheable_attribute: AttributeCache(attr, self)
-                for cacheable_attribute, attr in create_cache_map().items()
+                cacheable_attribute: AttributeCache(attr, self) for cacheable_attribute, attr in get_cache_map().items()
             }
             device_cache[device_info.device.duid] = cache
         self.cache: dict[CacheableAttribute, AttributeCache] = cache
-        self._listeners: list[Callable[[CacheableAttribute, RoborockBase], None]] = []
+        self._listeners: list[Callable[[str, CacheableAttribute, RoborockBase], None]] = []
 
     def __del__(self) -> None:
         self.release()
@@ -256,18 +263,21 @@ class RoborockClient:
                                     value[data_protocol.name] = data_point
                                     status = _cls.from_dict(value)
                                     for listener in self._listeners:
-                                        listener(CacheableAttribute.status, status)
+                                        listener(self.device_info.device.duid, CacheableAttribute.status, status)
                                 elif data_protocol in ROBOROCK_DATA_CONSUMABLE_PROTOCOL:
                                     if self.cache[CacheableAttribute.consumable].value is None:
                                         self._logger.debug(
-                                            f"Got consumable update({data_protocol.name}) before get_status was called."
+                                            f"Got consumable update({data_protocol.name})"
+                                            + "before get_consumable was called."
                                         )
                                         self.cache[CacheableAttribute.consumable]._value = {}
                                     value = self.cache[CacheableAttribute.consumable].value
                                     value[data_protocol.name] = data_point
                                     consumable = Consumable.from_dict(value)
                                     for listener in self._listeners:
-                                        listener(CacheableAttribute.consumable, consumable)
+                                        listener(
+                                            self.device_info.device.duid, CacheableAttribute.consumable, consumable
+                                        )
                                 return
                             except ValueError:
                                 pass
@@ -309,10 +319,8 @@ class RoborockClient:
             await self.async_disconnect()
         await self.async_connect()
 
-    async def _async_response(self, request_id: int, protocol_id: int = 0) -> tuple[Any, VacuumError | None]:
+    async def _wait_response(self, request_id: int, queue: RoborockFuture) -> tuple[Any, VacuumError | None]:
         try:
-            queue = RoborockFuture(protocol_id)
-            self._waiting_queue[request_id] = queue
             (response, err) = await queue.async_get(QUEUE_TIMEOUT)
             if response == "unknown_method":
                 raise UnknownMethodError("Unknown method")
@@ -322,6 +330,13 @@ class RoborockClient:
         finally:
             self._waiting_queue.pop(request_id, None)
 
+    def _async_response(
+        self, request_id: int, protocol_id: int = 0
+    ) -> Coroutine[Any, Any, tuple[Any, VacuumError | None]]:
+        queue = RoborockFuture(protocol_id)
+        self._waiting_queue[request_id] = queue
+        return self._wait_response(request_id, queue)
+
     def _get_payload(
         self,
         method: RoborockCommand,
@@ -368,18 +383,22 @@ class RoborockClient:
         params: Optional[list | dict] = None,
         return_type: Optional[Type[RT]] = None,
     ) -> RT:
-        parsed_method = parse_method(method)
+        cacheable_attribute_result = find_cacheable_attribute(method)
+
         cache = None
-        if parsed_method is not None:
-            cache = self.cache[parsed_method.attribute]
+        command_type = None
+        if cacheable_attribute_result is not None:
+            cache = self.cache[cacheable_attribute_result.attribute]
+            command_type = cacheable_attribute_result.type
+
         response: Any = None
-        if cache is not None:
-            if parsed_method.type == CommandType.GET:
-                response = await cache.async_value()
-            elif parsed_method.type == CommandType.SET:
-                response = await cache.update_value(params)
+        if cache is not None and command_type == CommandType.GET:
+            response = await cache.async_value()
         else:
             response = await self._send_command(method, params)
+            if cache is not None and command_type == CommandType.CHANGE:
+                await cache.refresh_value()
+
         if return_type:
             return return_type.from_dict(response)
         return response
@@ -440,7 +459,7 @@ class RoborockClient:
                 DustCollectionMode | WashTowelMode | SmartWashParams | None,
             ]
         ] = [self.get_dust_collection_mode()]
-        if dock_type == RoborockDockTypeCode.empty_wash_fill_dock or dock_type == RoborockDockTypeCode.s8_dock:
+        if dock_type in WASH_N_FILL_DOCK:
             commands += [
                 self.get_wash_towel_mode(),
                 self.get_smart_wash_params(),
