@@ -245,7 +245,7 @@ class RoborockDockTypeCode(RoborockEnum):
     auto_empty_dock_pure = 5
     s7_max_ultra_dock = 6
     s8_dock = 7
-    q_revo = 8
+    p10_dock = 8
 
 
 class RoborockDockDustCollectionModeCode(RoborockEnum):
